package com.jz.dmp.cmdexectool.scheduler.common.task.sync;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.ui.freemarker.FreeMarkerConfigurationFactoryBean;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.constant.DatasouceTypeConstant;
import com.jz.dmp.cmdexectool.common.utils.EncryptionUtils;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.entity.DmpSyncingDatasource;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.enums.MyDbType;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import com.jz.dmp.cmdexectool.scheduler.dao.datasource.MyBaseDataSource;

/**
 * @ClassName: SyncParameters
 * @Description: sync parameters
 * @Author : Bellamy
 * @Date 2021/2/25
 * @Version 1.0
 */
public class SyncParameters extends AbstractParameters {

    /**
     * shell script
     */
    private String script;

    private String taskAppId;

    /**
     * 数据源相关配置
     */
    private String source;

    /**
     * 目标相关配置
     */
    private String sink;

    /**
     * 环境相关配置
     */
    private String env;

    /**
     * ETL相关配置
     */
    private String transform;

    /**
     * waterdropScript脚本
     */
    private String waterdropScript;

    /**
     * resource list
     */
    private List<ResourceInfo> resourceList;

    /**
     * 前导语句
     */
    private List<String> preStatements;

    /**
     * 后导语句
     */
    private List<String> posStatements;

    /**
     * 源数据源
     */
    private MyBaseDataSource sourceBaseDataSource;

    /**
     * 目标数据源
     */
    private MyBaseDataSource targetBaseDataSource;

    public SyncParameters(String script, DmpProjectConfigInfoDto projectConfigInfoDto, DmpSyncingDatasourceDao dmpSyncingDatasourceDao, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String publicKey) throws Exception {
        source = "";
        env = "";
        sink = "";
        transform = "";
        this.script = script;
        JSONObject scriptObj = JSONObject.parseObject(script);
        Map<String, Object> scripts = (Map<String, Object>) scriptObj.get("scripts");
        Map<String, Object> sourceObj = (Map<String, Object>) scripts.get("reader"); //来源数据
        Map<String, Object> targetObj = (Map<String, Object>) scripts.get("writer"); //目标数据
        List<Map<String, Object>> mappingObj = (List<Map<String, Object>>) scripts.get("mappingRelation"); //字段映射关系
        //evn
        Map<String, String> envModel = new HashMap<String, String>();
        envModel.put("sparkappname", "Waterdrop");
        env = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfigurationFactoryBean);

        //target information
        Integer targetDbId = Integer.valueOf((String) targetObj.get("targetDbId"));
        DmpSyncingDatasource targetDatasource = dmpSyncingDatasourceDao.queryById(targetDbId);
        Integer targetTypeId = targetDatasource.getDatasourceType();
        String targetTable = String.valueOf(targetObj.get("targetTable"));//目标表

        //source information
        String sourceIds = (String) sourceObj.get("sourceDbId");
        String[] sourceIdArr = sourceIds.split(",");
        //Integer sourceId = Integer.valueOf(sourceIdArr[0]);
        String source_table_name = String.valueOf(sourceObj.get("registerTableName")); //来源表
        String[] registerTableName = source_table_name.split(",");

        //分库分表
        //***** source ******
        for (int i = 0; i < sourceIdArr.length; i++) {
            String id = sourceIdArr[i];
            DmpSyncingDatasource dmpSyncingDatasource = dmpSyncingDatasourceDao.queryById(Integer.valueOf(id));
            Integer sourceTypeId = dmpSyncingDatasource.getDatasourceType();

            //产生 Jdbc(MySQL、Oracle、SqlServer、PostgreSQL、Informix、DB2) source
            if ((sourceTypeId == DatasouceTypeConstant.MySQL
                    || sourceTypeId == DatasouceTypeConstant.Oracle
                    || sourceTypeId == DatasouceTypeConstant.SqlServer
                    || sourceTypeId == DatasouceTypeConstant.PostgreSQL
                    || sourceTypeId == DatasouceTypeConstant.Informix
                    || sourceTypeId == DatasouceTypeConstant.DB2)) {
                //source
                getJdbcSource(dmpSyncingDatasource, registerTableName, freeMarkerConfigurationFactoryBean, publicKey, i);
            }

            if (sourceTypeId == DatasouceTypeConstant.Hive) {
                //source
                getSourceHive(envModel, freeMarkerConfigurationFactoryBean, registerTableName, i);
            }
            if (sourceTypeId == DatasouceTypeConstant.Kudu) {
                //source
            }
            if (sourceTypeId == DatasouceTypeConstant.SFTP) {
                //source
                getSourceSftp(registerTableName, dmpSyncingDatasource, publicKey, sourceObj, freeMarkerConfigurationFactoryBean, i);
            }
            if (sourceTypeId == DatasouceTypeConstant.Elasticsearch) {
                //source
                getsourceElasticsearch(registerTableName, dmpSyncingDatasource, freeMarkerConfigurationFactoryBean, i);
            }
        }

        //产生 Jdbc(MySQL、Oracle、SqlServer、PostgreSQL、Informix、DB2) sink
        if ((targetTypeId == DatasouceTypeConstant.MySQL
                || targetTypeId == DatasouceTypeConstant.Oracle
                || targetTypeId == DatasouceTypeConstant.SqlServer
                || targetTypeId == DatasouceTypeConstant.PostgreSQL
                || targetTypeId == DatasouceTypeConstant.Informix
                || targetTypeId == DatasouceTypeConstant.DB2)) {
            getJdbcSink(targetDatasource, targetObj, freeMarkerConfigurationFactoryBean, publicKey, source_table_name);
        }
        //transform
        if (mappingObj.size() > 0 && null != mappingObj) {
            String sourceField = "";
            String targetField = "";
            for (Map<String, Object> item : mappingObj) {
                sourceField += "," + item.get("sourceField");
                targetField += "," + item.get("sourceField") + " as " + item.get("targetField");
            }
            Map sqlModel = new HashMap();
            StringBuilder sql = new StringBuilder()
                    .append(" select ")
                    .append(" " + sourceField.substring(1) + " ")
                    .append(" from ")
                    .append(" $t ");
            sqlModel.put("sql", sql);
            sqlModel.put("source_table_name", source_table_name);
            transform = transform + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, sqlModel, freeMarkerConfigurationFactoryBean);

            Map targetModel = new HashMap();
            StringBuilder targetSql = new StringBuilder()
                    .append(" select ")
                    .append(" " + targetField.substring(1) + " ")
                    .append(" from ")
                    .append(" $t ");
            targetModel.put("sql", targetSql);
            targetModel.put("source_table_name", source_table_name);
            transform = transform + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, targetModel, freeMarkerConfigurationFactoryBean);

        }

        //waterdrop script
        Map<String, String> waterdropModel = new HashMap<String, String>();
        waterdropModel.put("env", env);
        waterdropModel.put("source", source);
        waterdropModel.put("transform", transform);
        waterdropModel.put("sink", sink);
        waterdropScript = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL, waterdropModel, freeMarkerConfigurationFactoryBean);
    }

    private void getsourceElasticsearch(String[] registerTableName, DmpSyncingDatasource dmpSyncingDatasource, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, int i) {
        //source
        String tableName = registerTableName[i];
        //for (String tableName : registerTableName) {
        List<String> list = new ArrayList<>();
        String hosts = dmpSyncingDatasource.getHost() + ":" + dmpSyncingDatasource.getPort();
        list.add(hosts);
        Map jdbcModel = new HashMap();
        jdbcModel.put("hosts", list);
        jdbcModel.put("result_table_name", tableName);
        //jdbcModel.put("index", "");
        //jdbcModel.put("name_age", "");
        source = source + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_ELASTICSEARCH, jdbcModel, freeMarkerConfigurationFactoryBean);
        //}
    }

    public void getSourceHive(Map<String, String> envModel, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String[] registerTableName, int i) {
        //evn
        envModel.put("sparkSqlCatalogImplementation", "hive");
        env = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfigurationFactoryBean);
        //source
        String tableName = registerTableName[i];
        //for (String tableName : registerTableName) {
        Map hiveModel = new HashMap();
        hiveModel.put("pre_sql", " select * from  " + tableName);
        hiveModel.put("result_table_name", tableName);
        source = source + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, hiveModel, freeMarkerConfigurationFactoryBean);
        //}
    }

    public void getJdbcSource(DmpSyncingDatasource dmpSyncingDatasource, String[] registerTableName, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String publicKey, int i) {
        //int tableLength =registerTableName.length-1;
        String tableName = registerTableName[i];
        //for (String tableName : registerTableName) {
        Map jdbcModel = new HashMap();
        jdbcModel.put("driver", dmpSyncingDatasource.getDriverClassName());
        jdbcModel.put("url", dmpSyncingDatasource.getJdbcUrl());
        jdbcModel.put("table", tableName);
        jdbcModel.put("result_table_name", tableName);
        jdbcModel.put("user", dmpSyncingDatasource.getUserName());
        jdbcModel.put("password", EncryptionUtils.decode(dmpSyncingDatasource.getPassword(), publicKey));
        //jdbcModel.put("partitionColumn", "");
        //jdbcModel.put("numPartitions", "");
        //jdbcModel.put("lowerBound", "");
        //jdbcModel.put("upperBound", "");
        source = source + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, jdbcModel, freeMarkerConfigurationFactoryBean);
        //}
    }

    public void getJdbcSink(DmpSyncingDatasource targetDatasource, Map<String, Object> targetObj, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String publicKey, String source_table_name) {
        String postImportStatement = String.valueOf(targetObj.get("postImportStatement")); //导入后语句
        String preImportStatement = String.valueOf(targetObj.get("preImportStatement")); //导入前语句
        preStatements = new ArrayList<String>();
        preStatements.add(preImportStatement);
        posStatements = new ArrayList<String>();
        posStatements.add(postImportStatement);

        targetBaseDataSource = new MyBaseDataSource();
        targetBaseDataSource.setJdbcUrlDirect(targetDatasource.getJdbcUrl());
        targetBaseDataSource.setUser(targetDatasource.getUserName());
        String password = EncryptionUtils.decode(targetDatasource.getPassword(), publicKey);
        MyDbType myDbType = MyDbType.obtainByIdStr(targetDatasource.getDatasourceType().toString());
        targetBaseDataSource.setPassword(password);
        targetBaseDataSource.setMyDbType(myDbType);

        //sink
        Map jdbcSinkModel = new HashMap();
        jdbcSinkModel.put("driver", targetDatasource.getDriverClassName());
        jdbcSinkModel.put("url", targetDatasource.getJdbcUrl());
        //# 存储模式，支持overwrite、append、update、ignore、error
        jdbcSinkModel.put("save_mode", targetObj.get("targetInsertMergeOverwrite"));
        //当存储模式是 overwrite时，仅清除表中数据
        if (null != targetObj.get("targetInsertMergeOverwrite")) {
            if ("overwrite".equals(targetObj.get("targetInsertMergeOverwrite"))) {
                jdbcSinkModel.put("truncate", "true");
            } else {
                jdbcSinkModel.put("truncate", "false");
            }
        }
        jdbcSinkModel.put("dbtable", targetObj.get("targetTable")); //目标表
        jdbcSinkModel.put("user", targetDatasource.getUserName());
        jdbcSinkModel.put("password", password);
        jdbcSinkModel.put("source_table_name", source_table_name);
        sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_JDBC, jdbcSinkModel, freeMarkerConfigurationFactoryBean);
    }

    public void getSourceSftp(String[] registerTableName, DmpSyncingDatasource dmpSyncingDatasource, String publicKey, Map<String, Object> sourceObj, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, int i) {
        String tableName = registerTableName[i];
        //for (String tableName : registerTableName) {
        Map sftpModel = new HashMap();
        sftpModel.put("host", dmpSyncingDatasource.getHost()); //主机名
        sftpModel.put("user", dmpSyncingDatasource.getUserName()); //用户
        sftpModel.put("password", EncryptionUtils.decode(dmpSyncingDatasource.getPassword(), publicKey));
        sftpModel.put("port", dmpSyncingDatasource.getPort()); //端口
        if (null != sourceObj.get("fileType")) {
            sftpModel.put("fileType", sourceObj.get("ftpFileType")); //文件类型
        }
        sftpModel.put("delimiter", sourceObj.get("sourceFtpCsvDelimiter")); //分隔符
        sftpModel.put("head", "true"); //是否加载第一行
        sftpModel.put("multiLine", "true"); //多行解析
        if (null != sourceObj.get("sourceFtpDir")) {
            sftpModel.put("path", sourceObj.get("sourceFtpDir").toString()); //文件路径
        }
        sftpModel.put("result_table_name", tableName); //spark生成的临时表名
        source = source + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_SFTP, sftpModel, freeMarkerConfigurationFactoryBean);
        //}
    }

    @Override
    public boolean checkParameters() {
        return waterdropScript != null && !waterdropScript.isEmpty();
    }

    /**
     * get project resource files list
     *
     * @return resource files list
     */
    @Override
    public List<ResourceInfo> getResourceFilesList() {
        return resourceList;
    }

    public String getScript() {
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    public String getTaskAppId() {
        return taskAppId;
    }

    public void setTaskAppId(String taskAppId) {
        this.taskAppId = taskAppId;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getSink() {
        return sink;
    }

    public void setSink(String sink) {
        this.sink = sink;
    }

    public String getEnv() {
        return env;
    }

    public void setEnv(String env) {
        this.env = env;
    }

    public String getTransform() {
        return transform;
    }

    public void setTransform(String transform) {
        this.transform = transform;
    }

    public String getWaterdropScript() {
        return waterdropScript;
    }

    public void setWaterdropScript(String waterdropScript) {
        this.waterdropScript = waterdropScript;
    }

    public List<ResourceInfo> getResourceList() {
        return resourceList;
    }

    public void setResourceList(List<ResourceInfo> resourceList) {
        this.resourceList = resourceList;
    }
}
